-- This file is a part of Avoision.
-- 
-- Copyright (C) 2012 Jared Krinke.
-- 
-- Avoision is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

require("UI/Label.lua")
require("UI/Button.lua")
require("UI/Form.lua")

local arrowColor = Button.disabledColor
local arrowHoverColor = Button.focusedColor

Choice = { }

function Choice.new(text, choices, choiceChanged, skipCallbackArg)
    -- Determine maximum length of text first (so that left and right arrows stay in the same spot)
    local i = 1
    local maxCharacters = 0

    while i <= #choices do
        maxCharacters = max(maxCharacters, #choices[i])
        i = i + 1
    end

    local label = Button.new(text .. ": ")
    local leftArrow = Label.new("< ")
    local item = Label.new("", "center", false, maxCharacters)
    local rightArrow = Label.new(" >")
    local child = Form.newNestedFlow(3, leftArrow, item, rightArrow)
    local choice = Form.newNestedBox({ left = label, center = child })

    choice.child = child
    choice.choices = choices
    choice.choiceChanged = choiceChanged
    choice.choice = choice.choices[1]
    choice.item = item
    leftArrow:setColor(arrowColor)
    rightArrow:setColor(arrowColor)

    local skipCallback = false

    if isBoolean(skipCallbackArg) then
        skipCallback = true
    end

    function choice.setIndex(self, index, skipCallback)
        self.item:setText(self.choices[index])

        if index == 1 then
            leftArrow:setText("  ")
        elseif self.index == 1 then
            leftArrow:setText("< ")
        end

        if index == #self.choices then
            rightArrow:setText("  ")
        elseif self.index == #self.choices then
            rightArrow:setText(" >")
        end

        -- Re-layout the nested form containing the updated text
        self.child:layout()
        self.index = index
        self.choice = self.choices[self.index]

        if not isBoolean(skipCallback) or not skipCallback then
            if isFunction(self.choiceChanged) then
                self:choiceChanged(self.choice)
            end

            Audio.play("Sounds/Click.wav")
        end
    end

    function choice.select(self, choice, skipCallback)
        local i = 1

        while i <= #self.choices do
            if self.choices[i] == choice then
                self:setIndex(i, skipCallback)
                break
            end

            i = i + 1
        end
    end

    function choice.getChoice(self)
        return self.choice
    end

    function choice.movedLeft(self)
        if self.index > 1 then
            self:setIndex(self.index - 1)
        end

        return true
    end

    function choice.movedRight(self)
        if self.index < #self.choices then
            self:setIndex(self.index + 1)
        end

        return true
    end

    function choice.activated(self, source)
        local newIndex = (self.index % #self.choices) + 1

        -- Special case mouse input depending on what is being hovered over
        if source == "mouseButtonPressed" then
            local x, y = Mouse.getPosition()

            self:routePosition(x, y,
                function() newIndex = max(1, self.index - 1) end,
                function() newIndex = min(#self.choices, self.index + 1) end,
                function() end,
                function() newIndex = self.index end)
        end

        if newIndex ~= self.index then
            self:setIndex(newIndex)
        end

        self:updateMouseHighlight()
    end

    function choice.routePosition(self, x, y, left, right, neither, outside)
        -- Check for the label
        local labelX1, labelY1 = label:getPosition()
        local labelWidth, labelHeight = label:getSize()

        if x >= labelX1 and x <= labelX1 + labelWidth and y <= labelY1 and y >= labelY1 - labelHeight then
            neither()
        else
            -- Check for the arrows
            local x1, y1 = self:getPosition()
            local width, height = self:getSize()

            if x >= x1 and x <= x1 + width and y <= y1 and y >= y1 - height then
                if x < ((rightArrow:getPosition() - leftArrow:getPosition()) / 2 + leftArrow:getPosition() + leftArrow:getSize()) then
                    left()
                else
                    right()
                end
            else
                outside()
            end
        end
    end

    function choice.updateMouseHighlight(self)
        local x, y = Mouse.getPosition()

        self:routePosition(x, y,
            function() self:addMouseHighlight(leftArrow, rightArrow) end,
            function() self:addMouseHighlight(rightArrow, leftArrow) end,
            function() self:removeMouseHighlight() end,
            function() self:removeMouseHighlight() end)
    end

    function choice.addMouseHighlight(self, highlightedArrow, otherArrow)
        highlightedArrow:setColor(arrowHoverColor)
        otherArrow:setColor(arrowColor)
    end

    function choice.removeMouseHighlight(self)
        leftArrow:setColor(arrowColor)
        rightArrow:setColor(arrowColor)
    end

    function choice.setActive(self, active)
        label:setActive(active)

        if active then
            item:setColor(Button.defaultColor)
        else
            item:setColor(Button.disabledColor)
        end
    end

    function choice.getActive(self)
        return label:getActive()
    end

    -- Highlight one of the arrows when the mouse hovers over it
    function choice.mouseMoved(self, x, y)
        if self:getActive() then
        self:updateMouseHighlight()

        return true
        else
            return false
        end
    end

    local focusedBase = choice.focused

    function choice.focused(self)
        focusedBase(self)
        self:updateMouseHighlight()
    end

    local unfocusedBase = choice.unfocused

    function choice.unfocused(self)
        unfocusedBase(self)
        self:removeMouseHighlight()
    end

    -- Set the index, but don't call the callback
    choice:setIndex(1, skipCallback)

    return choice
end

